﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using budgetMonitoringSystem.DeptUserMaintSvc;
using System.Reflection;

namespace budgetMonitoringSystem.Forms
{
    public partial class AssignDeptUserForm : System.Web.UI.Page
    {
        List<String> _sErrMessages = new List<String>();
        clsErrorMsg _errMsg = new clsErrorMsg();

        protected void Page_Load(object sender, EventArgs e)
        {
            int iTemp = 0;
            string sPageMode = "";

            _errMsg.BuildErrorDisplay(_sErrMessages, this.Master);

            if (!Page.IsPostBack)
            {
                if (Request.QueryString["id"] != null)
                    hidClassId.Value = Request.QueryString["id"];
                if (Request.QueryString["code"] != null)
                    hidClassCode.Value = Request.QueryString["code"];
                if (Request.QueryString["mode"] != null)
                    sPageMode = Request.QueryString["mode"];

                if (hidClassId.Value.Trim() == "")
                    _sErrMessages.Add("Classification ID was not specified in the request.");
                else if (Int32.TryParse(hidClassId.Value.Trim(), out iTemp) == false)
                    _sErrMessages.Add("Classification ID specified is not an integer.");
                else if (Convert.ToInt32(hidClassId.Value.Trim()) <= 0)
                    _sErrMessages.Add("Classification ID specified should be greater than zero.");
                
                if (hidClassCode.Value.Trim() == "")
                    _sErrMessages.Add("Classification Code was not specified in the request.");

                if (sPageMode.Trim() == "")
                    sPageMode = "UD";
                hidPageMode.Value = sPageMode;

                if (_sErrMessages.Count <= 0)
                {
                    ProjClassMaintSvc.ProjClassMaintSvc svc = new ProjClassMaintSvc.ProjClassMaintSvc();
                    ProjClassMaintSvc.dcProjClass info = new ProjClassMaintSvc.dcProjClass();

                    try
                    {
                        info.ClassId = Convert.ToInt32(hidClassId.Value.Trim());
                        info.ClassCode = hidClassCode.Value;
                        info = svc.GetRecord(info);
                        hidClassType.Value = info.ClassType;
                        hidClassDesc.Value = info.ClassDesc;
                        hidClassName.Value = info.ClassName;   
                     
                        if(info.ClassType != "D")
                            _sErrMessages.Add("This classification is not a Department");
                    }
                    catch (Exception ex)
                    {                        
                        _sErrMessages.Add(ex.Message);
                    }
                    finally
                    {
                        svc.Dispose();
                    }
                }

                DisplayMenuHierarchy(sPageMode);

                if (_sErrMessages.Count > 0)
                {
                    btnAssign.Enabled = false;
                    btnRemove.Enabled = false;
                    btnSave.Enabled = false;
                }
                else
                {
                    lblDeptCode.Text = hidClassName.Value.Trim() + ": " + hidClassCode.Value + " - " + hidClassDesc.Value;                    
                }

                if(_sErrMessages.Count > 0 )
                    _errMsg.BuildErrorDisplay(_sErrMessages, this.Master);
            }
            LoadRecord();
        }

        void DisplayMenuHierarchy(string sPageMode)
        {
            try
            {
                clsGlobal.DisplayMenuHierarchy(Request, sPageMode, this.Master);
            }
            catch (Exception ex)
            {
                _sErrMessages.Add(ex.Message);
                _errMsg.BuildErrorDisplay(_sErrMessages, this.Master);
            }
        }

        void LoadRecord()
        {
            int iRowCount = 0;
            string sUserId = "";
            bool bIncludeFlag = false;

            DeptUserMaintSvc.DeptUserMaintSvc svc = new DeptUserMaintSvc.DeptUserMaintSvc();
            DeptUserMaintSvc.dcDeptUser info = new dcDeptUser();
            DeptUserMaintSvc.dcInitDeptUser initData = new dcInitDeptUser();

            List<DeptUserMaintSvc.dcDeptUser> unassignedList = new List<dcDeptUser>();
            List<DeptUserMaintSvc.dcDeptUser> assignedList = new List<dcDeptUser>();

            try
            {
                info.ClassId = Convert.ToInt32(hidClassId.Value);
                info.ClassCode = hidClassCode.Value;
                if (!Page.IsPostBack)
                {
                    initData = svc.GetInitDeptUser(info);
                    Session.Add("initDeptUser", initData);
                    assignedList.AddRange(initData.DeptUserList);
                    unassignedList.AddRange(initData.UserList);
                }
                else
                {
                    initData = (DeptUserMaintSvc.dcInitDeptUser)Session["initDeptUser"];
                    assignedList.AddRange(initData.DeptUserList);
                    unassignedList.AddRange(initData.UserList);

                    iRowCount = grdAssigned.Rows.Count;
                    for (int i = 0; i < iRowCount; ++i)
                    {
                        sUserId = grdAssigned.Rows[i].Items[1].Value.ToString();
                        if (grdAssigned.Rows[i].Items[0].FindControl("chkInclude") != null)
                        {
                            bIncludeFlag = ((CheckBox)grdAssigned.Rows[i].Items[0].FindControl("chkInclude")).Checked;
                            foreach (DeptUserMaintSvc.dcDeptUser userInfo in assignedList)
                            {
                                if (userInfo.UserId == sUserId)
                                    userInfo.IncludeFlag = bIncludeFlag;
                            }
                        }
                    }

                    iRowCount = grdUnassigned.Rows.Count;
                    for (int i = 0; i < iRowCount; ++i)
                    {
                        sUserId = grdUnassigned.Rows[i].Items[1].Value.ToString();
                        if (grdUnassigned.Rows[i].Items[0].FindControl("chkInclude") != null)
                        {
                            bIncludeFlag = ((CheckBox)grdUnassigned.Rows[i].Items[0].FindControl("chkInclude")).Checked;
                            foreach (DeptUserMaintSvc.dcDeptUser userInfo in unassignedList)
                            {
                                if (userInfo.UserId == sUserId)
                                    userInfo.IncludeFlag = bIncludeFlag;
                            }
                        }
                    }
                }
              
                grdAssigned.ClearDataSource();
                grdUnassigned.ClearDataSource();                

                grdAssigned.DataSource = assignedList;
                grdAssigned.DataBind();

                grdUnassigned.DataSource = unassignedList;                
                grdUnassigned.DataBind();

                initData.DeptUserList = assignedList.ToArray();
                initData.UserList = unassignedList.ToArray();
                Session["initDeptUser"] = initData;
            }
            catch (Exception ex)
            {
                _sErrMessages.Add(ex.Message);
            }
            finally
            {
                svc.Dispose();
            }
        }

        void AssignUsers(bool bAssignUsers)
        {
            int iRowCount = 0, iClassId = 0;
            string sUserId = "", sUserName = "";
            bool bIncludeFlag = false;

            DeptUserMaintSvc.DeptUserMaintSvc svc = new DeptUserMaintSvc.DeptUserMaintSvc();
            DeptUserMaintSvc.dcDeptUser info = new dcDeptUser();
            DeptUserMaintSvc.dcInitDeptUser initData = new dcInitDeptUser();

            List<DeptUserMaintSvc.dcDeptUser> unassignedList = new List<dcDeptUser>();
            List<DeptUserMaintSvc.dcDeptUser> assignedList = new List<dcDeptUser>();

            try
            {
                initData = (DeptUserMaintSvc.dcInitDeptUser)Session["initDeptUser"];
                assignedList.AddRange(initData.DeptUserList);
                unassignedList.AddRange(initData.UserList);

                if (bAssignUsers == false)
                {
                    iRowCount = grdAssigned.Rows.Count;
                    for (int i = 0; i < iRowCount; ++i)
                    {
                        sUserId = grdAssigned.Rows[i].Items[1].Value.ToString();
                        sUserName = grdAssigned.Rows[i].Items[2].Value.ToString();
                        bIncludeFlag = ((CheckBox)grdAssigned.Rows[i].Items[0].FindControl("chkInclude")).Checked;

                        if (bIncludeFlag)
                        {
                            info = new dcDeptUser();

                            info.UserId = sUserId;
                            info.UserName = sUserName;
                            info.IncludeFlag = false;
                            unassignedList.Add(info);

                            foreach (DeptUserMaintSvc.dcDeptUser userInfo in assignedList)
                            {
                                if (userInfo.UserId == sUserId)
                                {
                                    info = userInfo;
                                    break;
                                }
                            }
                            
                            assignedList.Remove(info);
                        }   
                    }
                }
                else
                {
                    iRowCount = grdUnassigned.Rows.Count;
                    for (int i = 0; i < iRowCount; ++i)
                    {
                        sUserId = grdUnassigned.Rows[i].Items[1].Value.ToString();
                        sUserName = grdUnassigned.Rows[i].Items[2].Value.ToString();
                        bIncludeFlag = ((CheckBox)grdUnassigned.Rows[i].Items[0].FindControl("chkInclude")).Checked;

                        if (bIncludeFlag)
                        {
                            info = new dcDeptUser();

                            Int32.TryParse(hidClassId.Value, out iClassId);
                            info.ClassId = iClassId;
                            info.ClassCode = hidClassCode.Value;
                            info.UserId = sUserId;
                            info.UserName = sUserName;
                            info.IncludeFlag = false;
                            assignedList.Add(info);

                            foreach (DeptUserMaintSvc.dcDeptUser userInfo in unassignedList)
                            {
                                if (userInfo.UserId == sUserId)
                                {
                                    info = userInfo;
                                    break;
                                }
                            }

                            unassignedList.Remove(info);
                        }                        
                    }
                }                

                initData.DeptUserList = assignedList.ToArray();
                initData.UserList = unassignedList.ToArray();
                Session["initDeptUser"] = initData;

                grdUnassigned.Rows.Clear();
                grdAssigned.Rows.Clear();

                grdUnassigned.Controls.Clear();
                grdAssigned.Controls.Clear();                

                LoadRecord();
            }
            catch (Exception ex)
            {
                _sErrMessages.Add(ex.Message);
                _errMsg.BuildErrorDisplay(_sErrMessages, this.Master);
            }
            finally
            {
                grdAssigned.EnableDataViewState = true;
                grdUnassigned.EnableDataViewState = true;
                svc.Dispose();
            }
        }

        void TickRecords(BaseControls.BaseGrid grid, bool bIsChecked)
        {
            int iRowCount = 0;

            try
            {
                iRowCount = grid.Rows.Count;
                for (int i = 0; i < iRowCount; ++i)
                    ((CheckBox)grid.Rows[i].Items[0].FindControl("chkInclude")).Checked = bIsChecked;                
            }
            catch (Exception ex)
            {
                _sErrMessages.Add(ex.Message);
                _errMsg.BuildErrorDisplay(_sErrMessages, this.Master);
            }
        }

        protected void btnCancel_Click(object sender, EventArgs e)
        {
            string sClassCode = string.Empty, sClassType = string.Empty;
            int iClassId = 0;

            iClassId = Convert.ToInt32(hidClassId.Value);
            sClassCode = hidClassCode.Value;
            sClassType = hidClassType.Value;
            Response.Redirect("ProjClassForm.aspx?id=" + iClassId.ToString() + "&code=" + sClassCode, true); 
        }

        protected void btnSelectUnassigned_Click(object sender, EventArgs e)
        {
            TickRecords(grdUnassigned, true);
        }

        protected void btnSelectAssigned_Click(object sender, EventArgs e)
        {
            TickRecords(grdAssigned, true);
        }

        protected void btnDeselectUnassigned_Click(object sender, EventArgs e)
        {
            TickRecords(grdUnassigned, false);
        }

        protected void btnDeselectAssigned_Click(object sender, EventArgs e)
        {
            TickRecords(grdAssigned, false);
        }

        protected void btnAssign_Click(object sender, EventArgs e)
        {
            AssignUsers(true);
        }

        protected void btnRemove_Click(object sender, EventArgs e)
        {
            AssignUsers(false);
        }

        protected void btnSave_Click(object sender, EventArgs e)
        {
            int iRowCount = 0, iClassId = 0;
            string sUserId = "";

            DeptUserMaintSvc.DeptUserMaintSvc svc = new DeptUserMaintSvc.DeptUserMaintSvc();
            DeptUserMaintSvc.dcDeptUser info = new dcDeptUser();
            List<DeptUserMaintSvc.dcDeptUser> assignedList = new List<dcDeptUser>();

            try
            {               
                iRowCount = grdAssigned.Rows.Count;
                for (int i = 0; i < iRowCount; ++i)
                {
                    sUserId = grdAssigned.Rows[i].Items[1].Value.ToString();
                   
                    info = new dcDeptUser();
                    Int32.TryParse(hidClassId.Value, out iClassId);

                    info.ClassId = iClassId;
                    info.ClassCode = hidClassCode.Value;
                    info.UserId = sUserId;
                    info.CreatedBy = clsGlobal.UserId;
                    assignedList.Add(info);                        
                }

                svc.Insert(assignedList.ToArray());
                Response.Redirect("ProjClassForm.aspx?id=" + iClassId.ToString() + "&code=" + hidClassCode.Value, true); 
            }
            catch (Exception ex)
            {
                _sErrMessages.Add(ex.Message);
                _errMsg.BuildErrorDisplay(_sErrMessages, this.Master);
            }
            finally
            {
                svc.Dispose();
            }
        }
    }
}